a {
    color: inherit;
    text-decoration-color: inherit;
}

div.black_background {
    background-color: black;
    z-index: 10;
}

div.doc_container {
    display: flex;
    flex-direction: column;
}

div.page_container {
    padding-top: 1.5em;
}

div.table {
    border: 1px solid dimgray;
    display: table;
    z-index: 10;
}

/* this is really more of a cell than a column */
div.column {
    border-left: 1px solid dimgray;
    border-right: 1px solid dimgray;
    display: table-cell;
    overflow: hidden;
    padding-top: 2px;
    padding-bottom: 0px;
    padding-left: 1em;
    padding-right: 1em;
    text-overflow: ellipsis;
    word-wrap: break-word;
}

div.columnheader {
    border-bottom: 1px solid dimgray;
    padding-bottom: 5px;
    padding-top: 5px;
}

div.document_body_container {
    padding-top: 0.5em;
    padding-bottom: 0.5em;
    padding-left: 1em;
    padding-right: 1em;
}

div.document_panel_header {
    border-bottom-style: solid;
    border-bottom-width: 3px;
    padding-bottom: 5px;
    padding-left: 1em;
    padding-right: 1em;
    padding-top: 5px;
    position: sticky;
    top: 0px;
}

div.no_expand {
    display: inline-block;
    width: fit-content;
}

div.person_bio_panel {
    margin-bottom: 0.7em;
    margin-left: auto;
    margin-right: 0;
    width: fit-content;
}

div.sticky_person_bio_panel {
    margin-bottom: 0.7em;
    margin-left: auto;
    margin-right: 0;
    position: sticky;
    top: 0px;
    width: fit-content;
}

div.row {
    display: table-row;
}

div.table_container {
    background-color: black;
    max-width: 99%;
    width: 99%;
    z-index: 10;
    width: fit-content;
}

fieldset {
    /* border: 10px solid transparent;
    border-top-color: black; */
    background-color: black;
    box-sizing: border-box;
    grid-area: 1 / 1; /* first row, first column */
    /* padding: 20px; */
    width: inherit;
    z-index: 10;
}

legend {
    background-color: black;
    font-family: Menlo, 'DejaVu Sans Mono', consolas, 'Courier New', monospace;
    font-size: inherit;
    font-style: italic;
    margin-left: auto;
    padding: 0 0.35em;
    z-index: 10;
}

li {
    margin-top: 0px;
    margin-bottom: 0px;
    font-family: Menlo, 'DejaVu Sans Mono', consolas, 'Courier New', monospace;
}

pre {
    margin-top: 0px;
    margin-bottom: 0px;
    font-family: Menlo, 'DejaVu Sans Mono', consolas, 'Courier New', monospace;
}

ul.no_bullets {
    list-style-type: none;
    margin: 0;
    padding: 0;
}
