"""
Crypto and virtual currency / fintech related files
"""
from epstein_files.documents.config.categories.finance import GOLDMAN_INVESTMENT_MGMT
from epstein_files.documents.config.config_builder import whistleblower_cfg
from epstein_files.documents.config.doc_cfg import DEFAULT_TRUNCATE_TO, NO_TRUNCATE, SHORT_TRUNCATE_TO, DocCfg, EmailCfg
from epstein_files.people.names import *
from epstein_files.util.constant.strings import *
from epstein_files.util.helpers.string_helper import join_truthy, quote

BEN_LAWSKY_NYDFS = f'head of NY Dept of Financial Services {BEN_LAWSKY}'
HONEYCOMB_FUND = 'Honeycomb Asset Management (Spotify and Tencent investors)'
KYARA_FUND = f"Epstein crypto fund {KYARA_INVESTMENT}"
RIOT_BLOCKCHAIN_DESCRIPTION = 'RIOT Blockchain (FKA "Bioptix") is a sketchy bitcoin miner in Texas'
VALAR_FUND = f"{PETER_THIEL}'s {VALAR_VENTURES} fund"


# This category is automatically 'interesting', see OtherFile class
CRYPTO_CFGS = [
    DocCfg(id='031743', description=f'a few pages describing the internet as a "New Nation State" (Network State?)'),
    DocCfg(
        id='025663',
        author=GOLDMAN_INVESTMENT_MGMT,
        description=f"An Overview of the Current State of Cryptocurrencies and Blockchain",
        date='2017-11-15',
        duplicate_ids=['EFTA00793120'],
    ),
    # Alternative currencies
    EmailCfg(
        id='EFTA02021870',
        author=BARNABY_MARSH,
        date='2012-01-29 11:15:00',
        highlight_quote="I would like to see you alone to discuss alternative currencies",
        recipients=[JEFFREY_EPSTEIN],
    ),
    EmailCfg(
        id='EFTA00941810',
        author_reason='"chris" in reply, "cell:/email" in signature',
        recipients=[CHRISTINA_GALBRAITH],
        truncate_to=1_000,
    ),
    EmailCfg(id='EFTA01852443'),
    EmailCfg(id='EFTA01754301'),
    EmailCfg(id='EFTA02361956', author=LINDA_STONE, author_reason='signature'),
    EmailCfg(id='EFTA02575430', description='alternative currencies group'),
    EmailCfg(id='EFTA00660838', description=f"Epstein says his trip to Ivory Coast was amazing", truncate_to=NO_TRUNCATE),
    EmailCfg(id='EFTA02391375', highlight_quote="Have you heard of the alternative currency"),
    # Bannon
    EmailCfg(id='030711', description='Epstein says "we can discuss michael and his coins", unclear what that means'),
    EmailCfg(id='026260', comment='Bannon cripto coin issues'),
    EmailCfg(id='026258', description=f'Bannon suggests {JEFFREY_WERNICK}, Parler COO and man affiliated with many shady crypto businesses, can help'),
    EmailCfg(id='030781', description="Esptein says the crypto coin issues they're having are 'US based'"),
    EmailCfg(id='026255', description='Epstein requests help with "coin issues" and "prohibitions foreign donor" (sic)', is_interesting=True),
    EmailCfg(id='EFTA02521522'),
    EmailCfg(id='EFTA02524752', description=f'{STEVE_BANNON} "is thinking of combining crypto with his politcial movement"', show_with_name=STEVE_BANNON),
    EmailCfg(id='EFTA01020383', description='Epstein emails Bannon a quote about the use of crypto to evade capital controls'),
    EmailCfg(id='EFTA00874793', description=f'Bannon says that they need to stop EU regulation of crypto scams "dead in its tracks"', is_interesting=True),
    EmailCfg(id='EFTA02517572', description=f'Bannon says that they need to stop EU regulation of crypto scams "dead in its tracks"'),
    EmailCfg(id='EFTA00881711', description=f"Bannon and Epstein discuss bitcoin and crypto donations", is_interesting=True),
    EmailCfg(id='EFTA00881759', description=f"Epstein and {STEVE_BANNON} discuss collection donations in bitcoin", is_interesting=True),
    # Bill Gates
    EmailCfg(id='EFTA02639760', description='Epstein pushing cryptocurrency on Bill Gates'),
    # Bitmain
    EmailCfg(id='EFTA01004576'),
    # Blockstream
    DocCfg(id='EFTA00797613', description='Alphabit crypto fund pitch deck', attached_to_email_id='EFTA00955063'),
    EmailCfg(id='EFTA02229342', author=AUSTIN_HILL),
    EmailCfg(
        id='EFTA02406818',
        author=AUSTIN_HILL,
        author_reason='unique email signature',
        description=f"{AUSTIN_HILL} asks Epstein about how the Treasury Dept. will treat dollar backed stablecoins like Tether",
    ),
    EmailCfg(
        id='EFTA00955063',
        description=f'{AUSTIN_HILL} explains that some crypto funds are engaged in fraudulent wash trading (the attached Excel sheet listing those funds has not been found)',
        is_interesting=True,
    ),
    EmailCfg(
        id='EFTA00673841',
        description="@pmarca is Marc Andreessen (a16z)",
        highlight_quote='business, Bitcoin, personal & esoteric mind games that we play',
        truncate_to=NO_TRUNCATE,
    ),
    EmailCfg(id='EFTA02529170', description=f"Blockstream set up a special company to invest in an Asian ride hailing service"),
    EmailCfg(id='EFTA01917402', description=f'discussion of Epstein investment in Blockstream funneled through {JOI_ITO}', is_interesting=True),
    EmailCfg(id='EFTA00999897', description=f"Epstein claims he's a fan of {ADAM_BACK} (who has recently claimed he had nothing to do with Epstein)"),
    EmailCfg(id='EFTA01915883', description=f"{JOI_ITO} doesn't trust {AUSTIN_HILL} (for good reason)"),
    EmailCfg(id='EFTA00988395', description=f'"Andy Back" is probably Blockstream bitcoin dev {ADAM_BACK}', duplicate_ids=['EFTA01928856']),
    EmailCfg(id='EFTA01925969', description=f"{AUSTIN_HILL} and {ADAM_BACK} and some women plan a trip to Epstein's island", is_interesting=True),
    EmailCfg(id='EFTA00929418', description=f"Epstein forwards information from {AUSTIN_HILL} about fraudulent crypto trading operations to {PAUL_BARRETT}"),
    EmailCfg(id='EFTA01010209', description=f"{AUSTIN_HILL} calls {AMIR_TAAKI} \"a bit crazy\'"),
    EmailCfg(id='EFTA01788532', description=f"{AUSTIN_HILL} asking about Jeffrey Katzenberg's contact info"),
    EmailCfg(id='EFTA02515798', recipients=[LINDA_STONE], description='Epstein confirms he invested in Blockstream'),  # response to EFTA02515756
    # Blockchain Capital / Brock Pierce / Crypto Currency Partners
    DocCfg(id='024271', author=BLOCKCHAIN_CAPITAL, description="pitch deck", date='2015-10-01'),
    EmailCfg(
        id='EFTA00689503',
        description=f"{BROCK_PIERCE} posts review of infamous Russian criminal crypto exchange BTC-e to LinkedIn",
        is_interesting=True,
        show_with_name=BROCK_PIERCE,
        # truncate_to=(1_400, 3_050),  # TODO: enable excerpts in emails
    ),
    DocCfg(id='EFTA01088644', author=BLOCKCHAIN_CAPITAL, description="pitch deck", date='2015-10-01'),
    DocCfg(id='EFTA00604942', author=BLOCKCHAIN_CAPITAL, description="Investor Update", date='2015-10-01'),
    DocCfg(id='EFTA01089535', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2014-10-01'),
    DocCfg(id='EFTA01089506', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2014-11-01'),
    DocCfg(id='EFTA01089500', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2015-01-01'),
    DocCfg(id='EFTA00621239', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2014-10-01'),
    DocCfg(id='EFTA01089526', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2014-08-01', attached_to_email_id='EFTA00997803'),
    DocCfg(id='EFTA01089485', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"investor report", date='2014-09-01', attached_to_email_id='EFTA00693621'),
    DocCfg(id='EFTA01093509', author=CRYPTO_CURRENCY_PARTNERS_II, description=f"partnership draft", truncate_to=DEFAULT_TRUNCATE_TO * 2),
    DocCfg(id='EFTA01093444', author=CRYPTO_CURRENCY_PARTNERS_II, description='application form', attached_to_email_id='EFTA00626220'),  #EFTA02708994
    DocCfg(id='EFTA01093471', author=CRYPTO_CURRENCY_PARTNERS_II, description='operating agreement', attached_to_email_id='EFTA00626220'),
    DocCfg(id='EFTA01126582', author=EXPRESSCOIN, description=f"pitch deck for {BROCK_PIERCE} co.", attached_to_email_id='EFTA00990186'),
    DocCfg(id='EFTA01125964', author=EXPRESSCOIN, description=f"pitch deck for {BROCK_PIERCE} co.", attached_to_email_id='EFTA00985076'),
    EmailCfg(id='EFTA02465656', description=f"{BROCK_PIERCE} describes investment in video game items company"),
    EmailCfg(id='EFTA00673123', description=f"{BROCK_PIERCE} and Epstein invested in Coinbase which would turn out to be extremely lucrative", duplicate_ids=['EFTA02371875']),
    EmailCfg(id='EFTA01051074', description=f'{BROCK_PIERCE} arranging meeting between Bannon and Epstein (in Doha?)'),
    EmailCfg(id='EFTA02361144', description=f'{BROCK_PIERCE} brings Epstein an opportunity to invest in Blockstream, bitcoin developer {ADAM_BACK}\'s company'),
    EmailCfg(id='EFTA00999392', description=f"{BROCK_PIERCE} discussing a possibly sham marriage to someone named Sue for immigration purposes"),
    EmailCfg(id='EFTA02514539', description=f"{BROCK_PIERCE} has tax problems (hasn't filed a tax return in 5 years)", is_interesting=True),
    EmailCfg(id='EFTA00658457', description=f'{BROCK_PIERCE} says they will not invest in Bitfury', truncate_to=700),
    EmailCfg(id='EFTA00998595', description=f'{BROCK_PIERCE} and Epstein discuss structure of crypto investments'),
    EmailCfg(id='EFTA01000882', description=f"{BROCK_PIERCE} asks for Epstein's permission to disclose his name to Coinbase team", is_interesting=True),
    EmailCfg(id='EFTA00664424', description=f"{BROCK_PIERCE} suggests {LARRY_SUMMERS} could get involved with Tether and Noble Bank"),
    EmailCfg(id='EFTA01139067', description=f'Epstein tells Farkas he has not invested in {CRYPTO_CURRENCY_PARTNERS_II}', is_interesting=True),
    EmailCfg(id='EFTA00752383', description=f"Mindshift conference attendee list including {BROCK_PIERCE}", is_interesting=True),
    EmailCfg(id='EFTA00626220', description=f'setting up "sidecar fund" with {BROCK_PIERCE} / {CRYPTO_CURRENCY_PARTNERS_II}'),
    # Boris Nikolic
    EmailCfg(id='EFTA00844780'),
    EmailCfg(id='EFTA01770220'),
    EmailCfg(id='EFTA02037504'),
    EmailCfg(id='EFTA00972689', description=f"Epstein and Nikolic ruminate on the Silk Road bust", truncate_to=NO_TRUNCATE),
    EmailCfg(
        id='EFTA02436105',
        description="one of Epstein's goals in life was to make a lot of money in virtual currency",
        is_interesting=True,
    ),
    # Coinbase
    DocCfg(
        id='EFTA01082451',
        author=BLOCKCHAIN_CAPITAL,
        date='2014-11-10',
        description=f'/ {COINBASE} nondisclosure agreement possibly regarding Epstein investment',
        is_interesting=True,
        truncate_to=2000,
    ),
    DocCfg(id='EFTA01401059', description=f"{DEUTSCHE_BANK} wire transfer receipt for Epstein's investment in {BLOCKSTREAM}"),
    DocCfg(id='EFTA01092555', author=COINBASE, description='certificate of incorporation', date='2013-12-12', truncate_to=SHORT_TRUNCATE_TO),
    DocCfg(id='EFTA01120975', author=COINBASE, description='Series B stock purchase', date='2013-12-12', truncate_to=DEFAULT_TRUNCATE_TO),
    DocCfg(id='EFTA01121035', author=COINBASE, description='Series C preferred stock purchase', date='2014-10-24'),
    EmailCfg(id='EFTA02606117', comment='Coinbase valuation'),
    EmailCfg(id='EFTA02608112', comment='Coinbase valuation'),
    EmailCfg(id='EFTA00901772', description=f"Epstein's investment in {COINBASE} would earn him most of $100 million", is_interesting=True),
    EmailCfg(id='EFTA00900908', description="negotiation of repurchase of part of Epstein's stake in Coinbase"),
    # Crypto PR Lab
    DocCfg(id='EFTA00295126', author=CRYPTO_PR_LAB, description="accounting statement", show_full_panel=True),
    DocCfg(id='EFTA01299820', author=CRYPTO_PR_LAB, description="bank transfer", date='2018-12-18'),
    DocCfg(id='EFTA00309271', author=CRYPTO_PR_LAB, description='financial statement'),
    DocCfg(id='EFTA01613759', author=CRYPTO_PR_LAB, description="letter of intent of acquisitionfrom Transform Group"),
    DocCfg(id='EFTA01613762', author=CRYPTO_PR_LAB, description=f"WhatsApp convo with {MARIA_PRUSAKOVA}", date='2019-05-21'),
    DocCfg(id='EFTA01612721', author=CRYPTO_PR_LAB, description=f"WhatsApp convo with {MARIA_PRUSAKOVA}", date='2019-05-21'),
    EmailCfg(id='033255', description=f"{MASHA_DROKOVA}'s friend Dylan Love suggests he could quote Epstein explaining \"proper implementation of crypto might solve financial corruption issues\""),
    EmailCfg(id='EFTA01013266', description=f"{MARIA_PRUSAKOVA}'s {CRYPTO_PR_LAB} request for payment for Davos", is_interesting=True),
    EmailCfg(id='EFTA02285514', description=f"Medici Bank and {MARIA_PRUSAKOVA} meeting", is_interesting=True),
    EmailCfg(id='EFTA01013922', description=f"{CRYPTO_PR_LAB} business plan", is_interesting=True),
    EmailCfg(id='EFTA00495349', author=MARIA_PRUSAKOVA, author_reason=CRYPTO_PR_LAB, description='Ed Boyle / Medici Bank'),
    EmailCfg(
        id='EFTA02267133',
        author=LESLEY_GROFF,
        recipients=[MARIA_PRUSAKOVA],
        recipient_uncertain=f"{CRYPTO_PR_LAB} signature, could be Prusakova's partner",
        truncate_to=NO_TRUNCATE
    ),
    EmailCfg(
        id='EFTA02349697',
        author=MARIA_PRUSAKOVA,
        author_reason='boyfriend Christian, same as 032374',
        description=f'{MARIA_PRUSAKOVA} says she is starting a crypto company with {ZUBAIR_KHAN}',
    ),
    # Crypto PR Lab / Medici Bank
    DocCfg(id='EFTA01613731', description='Medici Bank conversation on Whatsapp'),
    DocCfg(id='EFTA00805569', description='Mercantile Global Holdings, San Juan Mercantile Bank & Trust, Noble Bank + Signature Bank + BitGo'),
    DocCfg(id='EFTA01088079', description="someone's thoughts on bitcoin"),
    EmailCfg(id='EFTA00495372', description=f'discussion of Medici Bank, a new successor to Noble Bank as a crypto bank', is_interesting=True),
    EmailCfg(
        id='EFTA01030070',
        description="Bo Collins inspired by Epstein to set up Mercantile Global Holdings crypto trading offering stablecoins to people who can't get bank accounts",
        is_interesting=True,
    ),
    EmailCfg(
        id='EFTA02603577',
        description='meeting with Mercantile Global Holdings Bo Collins, who would testify in the trial of Miles Guo',
        is_interesting=True
    ),
    # David Stern
    EmailCfg(
        id='EFTA02494305',
        description=f"planning a crypto exchange in Belarus, {DAVID_STERN}'s \"Chinese guys\" can't do it because of the PRC's crackdown on crypto",
    ),
    EmailCfg(id='EFTA00886763', description=f"documents are being prepared to license Epstein and {DAVID_STERN}'s crypto exchange in Belarus"),
    EmailCfg(
        id='EFTA02494000',
        description=f'Epstein says a crypto exchange in Belarus could be "very useful", {DAVID_STERN}\'s Chinese guys could have brought "significant volume"',
        is_interesting=True,
    ),
    # Honeycomb
    DocCfg(id='EFTA00803491', author=HONEYCOMB_FUND, description="deck", is_interesting=True),
    DocCfg(id='EFTA00803459', author=HONEYCOMB_FUND, description="January 2019 report", is_interesting=True),
    DocCfg(id='EFTA00603445', author=HONEYCOMB_FUND, description="July 2017 report", is_interesting=True),
    DocCfg(id='EFTA00803464', author=HONEYCOMB_FUND, description="July 2018 report", date='2018-07-17', is_interesting=True),
    # Howard Lutnick
    DocCfg(id='EFTA01582043', description=f"{DEUTSCHE_BANK} / {CANTOR} transaction"),
    DocCfg(id='EFTA01230639', date='2012-12-22', date_uncertain=True, show_full_panel=True),
    DocCfg(id='EFTA00434306', description='calendar reminder for call with Howard Lutnick', date='2011-04-26', show_full_panel=True),
    DocCfg(
        id='EFTA00020515',
        author=FBI,
        description='tip about Howard Lutnick and financial irregularities',
        is_interesting=True,
        show_full_panel=True,
    ),
    DocCfg(
        id='EFTA00289560',
        date='2012-12-28',
        description=f'{HOWARD_LUTNICK} and Epstein business deal for Adfin Solutions, Inc. (Southern Trust is Epstein)',
        truncate_to=(160_000, 162_000),
    ),
    DocCfg(
        id='EFTA01249210',
        author=FBI,
        description="tip linking American commerce secretary and Tether asset custodian Howard Lutnick to Ponzi schemes and Russian money",
        is_interesting=True,
        show_with_name=BROCK_PIERCE,
    ),
    DocCfg(
        id='EFTA01733746',
        date='2011-05-18',
        description=f'schedule showing "drinks" with {HOWARD_LUTNICK}',
        is_valid_for_name_scan=False,
        show_full_panel=True,
        truncate_to=700,
    ),
    DocCfg(
        id='EFTA01249207',
        author=FBI,
        description="tip linking American commerce secretary and Tether asset custodian Howard Lutnick to Ponzi schemes and Russian money",
        show_full_panel=True,
        show_with_name=BROCK_PIERCE,
    ),
    DocCfg(
        id='EFTA01684300',
        author=FBI,
        description=f'evidence list w/tip about {HOWARD_LUTNICK}, Cantor Fitzgerald, and BGC from an employee who had already blown the whistle successfully',
        truncate_to=(77_000, 77_300),
    ),
    EmailCfg(
        id='EFTA00173881',
        author='US Secret Service',
        recipients=['Enterprise Vetting Center'],
        show_with_name=HOWARD_LUTNICK,
        truncate_to=800
    ),
    EmailCfg(
        id='EFTA00873541',
        description=f"Epstein says his neighbor {HOWARD_LUTNICK} (recently seen with {PRINCE_ANDREW}) is smart",
        show_with_name=HOWARD_LUTNICK,
    ),
    EmailCfg(id='EFTA02374900', description=f'article about {HOWARD_LUTNICK} real estate purchase'),
    EmailCfg(id='EFTA00685520', description=f'call with {HOWARD_LUTNICK}'),
    EmailCfg(id='EFTA00436468', description=f'call with {HOWARD_LUTNICK} after he goes overseas', truncate_to=NO_TRUNCATE),
    EmailCfg(id='EFTA00462693', description=f"Epstein gets a table for 10 at the dinner honouring {HOWARD_LUTNICK}"),
    EmailCfg(id='EFTA00398744', description=f"{HOWARD_LUTNICK} thanks Epstein for the island vacay"),
    EmailCfg(id='EFTA00474452', description=f"{HOWARD_LUTNICK} and {JEFFREY_EPSTEIN}, NIMBYs"),
    EmailCfg(id='EFTA00398719', description=f'{HOWARD_LUTNICK} visit to the island', truncate_to=2500),
    EmailCfg(id='EFTA00401065', description=f'{HOWARD_LUTNICK} visit to the island', truncate_to=NO_TRUNCATE),
    EmailCfg(id='EFTA00398853', description=f'{HOWARD_LUTNICK} visit to the island', truncate_to=NO_TRUNCATE),
    EmailCfg(id='EFTA00873540', description=f'{HOWARD_LUTNICK} and {PRINCE_ANDREW}'),
    EmailCfg(id='EFTA00443752', description=f"invitation from {HOWARD_LUTNICK}"),
    EmailCfg(id='EFTA00957552', description=f'thank you note for a $50,000 donation in honor of Tether banker {HOWARD_LUTNICK}'),
    EmailCfg(id='EFTA00970606', show_with_name=HOWARD_LUTNICK),
    EmailCfg(id='EFTA01778423', show_with_name=HOWARD_LUTNICK),
    # Jem Bendell
    EmailCfg(
        id='EFTA01005603',
        author=JEM_BENDELL,
        author_reason='sig',
        highlight_quote="if you are interested in blockchain and alternative currencies",
    ),
    EmailCfg(id='EFTA00630785', description='alternative currencies discussion'),
    EmailCfg(id='EFTA02724230', description='alternative currencies discussion', truncate_to=NO_TRUNCATE),
    # Jeremy Rubin
    EmailCfg(id='EFTA02466979', description='"AID Coin"'),
    EmailCfg(id='EFTA02343669', description='rumours that people are using bitcoin to exfiltrate capital from China'),
    # Joi Ito / Kyara
    DocCfg(id='024256', author=JOI_ITO, description=f"Internet & Society: The Technologies and Politics of Control"),
    EmailCfg(id='025598', is_interesting=False),
    EmailCfg(id='028847', description=f"the announcement of a hunt for Putin's money seems to greatly concern Epstein and {JOI_ITO}"),
    EmailCfg(id='026360', description=f"{JOI_ITO} writes to Epstein to thank him for providing funding for the bitcoin core development team"),
    EmailCfg(id='025603', description=f"Epstein funnels money from {ANDREW_FARKAS} to {MIT_MEDIA_LAB} / {JOI_ITO}", truncate_to=850),
    DocCfg(id='EFTA00805860', description=f"{JOI_ITO}'s Neoteny 3 fund investor update / portfolio"),
    DocCfg(id='EFTA01118268', description=f"{JOI_ITO}'s Neoteny 3 fund investor update", date='2015-06-30'),
    DocCfg(id='EFTA01116280', description=f"{KYARA_FUND} I LLC (Physical Graph Corporation) operating agreement with edits"),
    DocCfg(id='EFTA01108775', description=f"{KYARA_FUND} I LLC (Physical Graph Corporation) operating agreement"),
    DocCfg(id='EFTA00605699', description=f"{KYARA_FUND} II LLC incorporation papers"),
    DocCfg(id='EFTA01186455', description=f"{KYARA_FUND} II LLC operating agreement"),
    DocCfg(id='EFTA00584696', description=f"{KYARA_FUND} III LLC operating agreement"),
    EmailCfg(
        id='EFTA02376268',
        description=f"introduction to {JEREMY_RUBIN}",
        recipients=[JEFFREY_EPSTEIN, JEREMY_RUBIN, LINDA_STONE, None],
    ),
    EmailCfg(id='EFTA02342561', description=f"China, {REID_HOFFMAN}, bitcoin", is_interesting=True),
    EmailCfg(id='EFTA02601259', description=f"discussion of investments hidden through {JOI_ITO}"),
    EmailCfg(id='EFTA00989593', description=f'donations from {LEON_BLACK} to {MIT_MEDIA_LAB} / Digital Currency Initiative {QUESTION_MARKS}'),
    EmailCfg(id='EFTA00999549', description=f"{JOI_ITO} and {JEREMY_RUBIN} meet {LARRY_SUMMERS} to discuss bitcoin", is_interesting=True),
    EmailCfg(id='EFTA01752601', description=f"{JOI_ITO} and Epstein name their new fund Kyara"),
    EmailCfg(id='EFTA02703885', author=LINDA_STONE, author_reason='sig', description=f"{LINDA_STONE} introducing Epstein to {JEREMY_RUBIN}"),
    EmailCfg(id='EFTA02505439', description=f'summary of {KYARA_INVESTMENT} shell companies with {JOI_ITO}'),
    EmailCfg(id='EFTA01973301', recipients=[LINDA_STONE], author_reason='signature'),
    EmailCfg(id='EFTA00473175', truncate_to=1000),
    # Kushner
    DocCfg(id='EFTA00128987', description='suspicious activity report (SAR) about Kushner co. crypto payments to suspicious Russian person'),
    # Masha Drokova
    EmailCfg(
        id='EFTA01003115',
        author=MASHA_DROKOVA,
        author_reason='"masha" in quoted',
        description='"Dylan" is probably Dylan Love who wrote a story about Epstein and bitcoin arranged by Masha Drokova',
        truncate_to=NO_TRUNCATE,
    ),
    EmailCfg(id='EFTA01010171'),
    EmailCfg(id='EFTA01010128'),
    # Misc
    DocCfg(
        id='EFTA02005667',
        author='Clearstone',
        date='2011-02-01',
        date_uncertain=True,
        description='Online Gaming Fund pitch deck',
        non_participants=[AUSTIN_HILL],
    ),
    DocCfg(
        id='EFTA01388197',
        attached_to_email_id='EFTA01388354',
        author=DEUTSCHE_BANK,
        date='2018-04-03',
        description='bitcoin and blockchain symposium',
    ),
    DocCfg(
        id='EFTA01074293',
        author='Jerry Brito, Houman Shadab, & Andrea Castillo',
        date='2014-04-10',
        description='discussion draft of "Bitcoin Financial Regulation" paper',
    ),
    DocCfg(
        id='EFTA00797937',
        author='John Pfeffer',
        date='2017-12-24',
        description="An (Institutional) Investor's Take on Cryptoassets",
    ),
    DocCfg(
        id='EFTA01612888',
        author='Kevin Peterson, Bammohan Deeduvanu, Pradip Kanjamala, and Kelly Boles',
        description='A Blockchain-Based Approach to Health Information Exchange Networks',
    ),
    DocCfg(
        id='EFTA02725909',
        date='2016-01-01',
        description=f'memo to NYDFS of NYC Bitcoin Exchange, Balaji Srinivisan & {ANDREW_FARKAS} on board',
        date_uncertain='mentions ethereum so must be after 2015-06',
    ),
    DocCfg(id='EFTA00811666', description='asset valuations of Epstein\'s holdings, includes "Coinbase via grat"', date='2018-01-31'),
    DocCfg(id='EFTA01734786', description='LedgerX Series B pitch deck'),
    DocCfg(id='EFTA00605996', description='Wedbush BUY rating on Digital Currency Group GBTC', is_interesting=False),
    EmailCfg(id='EFTA00867030', author='Kathy', description='discussion of a crypto token based on GDAX (whatever that means)', is_interesting=True),
    EmailCfg(id='EFTA01060993', description=f"Epstein says he knows \"a few\" guys who hold over $50 million in bitcoin", is_interesting=True),
    EmailCfg(id='EFTA01915234', description='discussion of crypto regulations', is_interesting=True),
    EmailCfg(id='EFTA01004753', description=f"founder of Layer1 is the only guy in crypto who doesn't want to meet Epstein (didn't last though)"),
    EmailCfg(id='EFTA02601503', description=f"discussion of investing in Layer1, an important crypto business"),
    EmailCfg(id='EFTA02517623', description=f"{VINCENZO_IOZZO} explains to Epstein how to deanonymize bitcoin", is_interesting=True),
    EmailCfg(id='EFTA00699275', description=f"'Fred' is probably Coinbase co-founder Fred Ehrsam"),
    EmailCfg(id='EFTA00830911', description='fundraising email for LedgerX which was later acquired by FTX for $298 million'),
    EmailCfg(id='EFTA01750652', description=f"discussion of using blockchains in the repo markets", is_interesting=True),
    EmailCfg(id='EFTA02002675', description=f"{JASON_CALACANIS} passes Epstein the names of the bitcoin core developers"),
    # Reid Hoffman
    EmailCfg(id='EFTA00820371', description='bank digital currencies article', is_interesting=True),
    # Russia
    EmailCfg(id='EFTA02591998', description='Epstein spoke to Vladimir Putin about digital currency', is_interesting=True),
    # SEC / NYDFS
    EmailCfg(id='EFTA00668932', description=f"pressing Farkas for a meeting with {BEN_LAWSKY_NYDFS}"),
    EmailCfg(id='EFTA01747752', description=f"Farkas delivering {BEN_LAWSKY_NYDFS}", truncate_to=700),
    EmailCfg(id='EFTA02588398', description=f"Epstein went to the Treasury Dept of the US to talk about bitcoin", is_interesting=True),
    EmailCfg(id='EFTA00987194', description=f"Epstein proposes {BEN_LAWSKY_NYDFS} get rid of sales tax on bitcoin", is_interesting=True),
    EmailCfg(id='EFTA02591315', description=f"{ANDREW_FARKAS} arranging for Epstein to meet {BEN_LAWSKY_NYDFS}", is_interesting=True, truncate_to=1900),
    EmailCfg(id='EFTA01002518', description=f"confirmation that Epstein did indeed meet {BEN_LAWSKY_NYDFS} to discuss a 'high yield product'"),
    EmailCfg(id='EFTA02587320', author=KATHRYN_RUEMMLER, description=f"Kathy Ruemmler, crypto bro-ette", is_interesting=True),
    EmailCfg(
        id='EFTA00357845',
        description=f"Epstein and {KATHRYN_RUEMMLER} meet with {BEN_LAWSKY_NYDFS} to discuss bitcoin",
        is_interesting=True,
        recipients=[LESLEY_GROFF],
        recipient_uncertain=True,
    ),
    # Sharia coin
    EmailCfg(id='032359', description='HBJ brings up "e-currency" (Sharia Coin, probably)'),
    EmailCfg(id='EFTA00964459', description=f'discussion of sharia compliant crypto token', is_interesting=True),
    EmailCfg(id='EFTA00990442', description=f"Epstein offers to cover all costs for Sharia Coin", is_interesting=True),
    EmailCfg(id='EFTA02396341', description='the Sharia Coin gambit', is_interesting=True),
    EmailCfg(id='EFTA01005117', description='announcement that the Stellar blockchain is Sharia compliant'),
    # Steven Sinofsky / a16z
    EmailCfg(id='EFTA01804670', highlight_quote='Andreessen really into this whole virtual money space', truncate_to=700),
    # Valar
    DocCfg(
        id='EFTA00591045',
        attached_to_email_id='EFTA01001339',
        author=VALAR_FUND,
        description='pitch deck',
        non_participants=[MASAYOSHI_SON],
    ),
    DocCfg(id='EFTA00810239', author=VALAR_FUND, description='pitch deck'),
    DocCfg(id='EFTA00810510', author=VALAR_FUND, description='Fall 2016 Update'),
    DocCfg(id='EFTA00810474', author=VALAR_FUND, description='Fall 2018 Update'),
    DocCfg(id='EFTA01121910', author=VALAR_VENTURES, description="contract", truncate_to=DEFAULT_TRUNCATE_TO),
    DocCfg(id='EFTA00808277', author=VALAR_VENTURES, description="contract", truncate_to=DEFAULT_TRUNCATE_TO),
    DocCfg(id='EFTA01088484', author=VALAR_VENTURES, description="contract", truncate_to=DEFAULT_TRUNCATE_TO),
    DocCfg(id='EFTA00591691', author=VALAR_VENTURES, description="contract", truncate_to=DEFAULT_TRUNCATE_TO),
    DocCfg(id='EFTA00810362', author=VALAR_VENTURES, description="investor questionnaire", truncate_to=DEFAULT_TRUNCATE_TO),
    EmailCfg(id='EFTA01001339', description=f"{PETER_THIEL} introduces Epstein to the {VALAR_VENTURES} founders"),
    # Vincenzo
    EmailCfg(id='EFTA00637023', description=f"discussion of getting around money laundering laws in places like Myanmar + Mongolia"),
    EmailCfg(id='EFTA02588723', description=f'discussion of crypto food stamps debit cards', is_interesting=True),
    EmailCfg(id='EFTA02588748', description=f'discussion of crypto food stamps debit cards', is_interesting=True, truncate_to=800),
    EmailCfg(id='EFTA02584771', description=f"discussion of decentralized prediction markets (e.g. Polymarket)", is_interesting=True),
    EmailCfg(id='EFTA01008242', description='Epstein passes on an investment in Radius'),
    EmailCfg(id='EFTA00995269', description=f"Epstein suggests Monaco and Vatican City as good places for crypto"),
    # ZCash / Madars Virza
    DocCfg(id='EFTA00811130', author=PERKINS_COIE, description='tax opinion on ZCash tokens'),
    DocCfg(id='EFTA00603348', description=f"Electric Coin Company created the untraceable crypto ZCash funded by {LARRY_SUMMERS}'s DCG"),
    DocCfg(id='EFTA00803405', description=f"Honeycomb Asset Management fund brochure", truncate_to=DEFAULT_TRUNCATE_TO),
    EmailCfg(id='EFTA02282357', author=DAPHNE_WALLACE, author_uncertain='could be Lesley'),
    EmailCfg(id='EFTA02283028', author=DAPHNE_WALLACE, author_uncertain='could be Lesley'),
    EmailCfg(id='EFTA02620980', description=f"{MADARS_VIRZA} says Tether was created to avoid anti-money laundering laws"),
    EmailCfg(id='EFTA01781741', description=f'{JOI_ITO} introduces Epstein to {MADARS_VIRZA} and the untraceable private crypto ZCash'),
    EmailCfg(id='EFTA02645742', description=f"tax implications of the untraceable 'privacy coin' ZCash", is_interesting=True),
    EmailCfg(
        id='EFTA02374960',
        description=f"Epstein says that he will fund a seat at MIT Media Lab for ZCash founder {MADARS_VIRZA}",
        is_interesting=True,
    ),
    # Unsorted
    EmailCfg(
        id='EFTA00096009',
        author='SDNY Cybercrimes',
        description='Epstein legal defense team requested bail hearing transcript of OneCoin fraudster Konstantin Ignatova',
        recipients=[USANYS],
    ),
    EmailCfg(id='EFTA00629471', author=JOHN_BROCKMAN, non_participants=[AUSTIN_HILL, 'David Geffen'], truncate_to=3000),
    EmailCfg(id='EFTA01762201', highlight_quote='indoctrinating kids into an economy', show_with_name=BOBBY_KOTICK),
    EmailCfg(id='EFTA02414991', description=f"Epstein signs up for World of Warcraft", truncate_to=350),
    EmailCfg(id='EFTA01388354', description=f"invitation to {DEUTSCHE_BANK} blockchain event with Mike Novogratz"),
    EmailCfg(id='EFTA01434500', description=f"everyone assumes Epstein's banker {PAUL_BARRETT} will want to know about the blockchain event"),
    EmailCfg(id='EFTA00993615', description=f"{MASHA_DROKOVA} explains the price of bitcoin can be manipulated if Epstein makes public comments"),
    EmailCfg(id='EFTA01784901', description=f"{JEREMY_RUBIN} has cashed multiple checks directly from Epstein", is_interesting=True),
    EmailCfg(
        id='EFTA01007544',
        description=f'{JEREMY_RUBIN} describes "grey area between pump and develop" when Epstein objects on ethical grounds',
        is_interesting=True,
    ),
    EmailCfg(
        id='EFTA01748990',
        description=f"{AUSTIN_HILL} is upset that Ito and Epstein seem to have invested in Stellar and Ripple, competing blockchain ecosystems",
        is_interesting=True,
    ),
    EmailCfg(id='EFTA01943067', description=f'comments on {ARIANE_DE_ROTHSCHILD} and bitcoin', is_interesting=True),
    EmailCfg(id='EFTA00835324', recipients=[SERGEY_BELYAKOV], description='assessment of Mycelium bitcoin wallet'),
    EmailCfg(id='EFTA00368659', description=f"meeting with {REID_HOFFMAN} about bitcoin", is_interesting=True),
    EmailCfg(id='EFTA01039626'),
    EmailCfg(
        id='EFTA02035756',
        description=f"{AL_SECKEL} introduces Epstein to Bannon business partner {BROCK_PIERCE} of Tether/{BLOCKCHAIN_CAPITAL}",
        is_interesting=True,
    ),
    EmailCfg(id='EFTA02372964', description='an "ICO" or "initial coin offering" was a very popular type of crypto scam at this time'),
    EmailCfg(
        id='EFTA02177147',
        author=LINDA_STONE,
        author_uncertain='"Hi Linda"',
        description=f"Epstein planning a symposium \"on Alternative Money or Complementary Currency\" in 2012",
        is_interesting=True,
        recipients=[LESLEY_GROFF],
    ),
    EmailCfg(
        id='EFTA01965506',
        author=LINDA_STONE,
        author_reason='"iPhone word substitution" in signature, which is traced back to "Linda, thanks" in EFTA00961792',
        description='Barry Silbert became the CEO of Digital Currency Group, a huge US crypto fund w/Larry Summers on the board',
        is_interesting=True,
    ),
    EmailCfg(id='EFTA00899331', truncate_to=NO_TRUNCATE),
]

# Chris Dilorio emails
WHISTLEBLOWER_IDS = {
    'EFTA00093702_37': '',
    'EFTA00093702_13': '',
    'EFTA00093702_21': RIOT_BLOCKCHAIN_DESCRIPTION,
    'EFTA00093702_30': f"Signature Bank failed in 2023, {RIOT_BLOCKCHAIN_DESCRIPTION}",
    'EFTA00093702_29': RIOT_BLOCKCHAIN_DESCRIPTION,
    'EFTA00093702_28': RIOT_BLOCKCHAIN_DESCRIPTION,
    'EFTA00093702_27': RIOT_BLOCKCHAIN_DESCRIPTION,
    'EFTA00093702_26': RIOT_BLOCKCHAIN_DESCRIPTION,
}


CRYPTO_CFGS.extend([whistleblower_cfg(id, desc) for id, desc in WHISTLEBLOWER_IDS.items()])
